<?php

/**
 * @file
 * Contains \Drupal\otp_verification\Form\MiniorangeSettings.
 */

namespace Drupal\otp_verification\Form;

use Drupal\Core\Form\FormBase;
use Drupal\user\Entity\User;
use Drupal\otp_verification\MiniorangeOtpUtilities;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\otp_verification\MiniorangeOtpVerificationConstants;
use Drupal\Component\Utility\Unicode;


class MiniorangeValidateUser extends FormBase
{
  public function getFormId()
  {
    return 'miniorange_otp_verification_validate_user';
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
  {

    global $base_url, $user, $form, $form_state;

    // DO NOT DELETE
    /*
    if (session_status() == PHP_SESSION_NONE) {
        session_start();
    }*/

    //check if url is hit directly without registration
    if (!isset($_SESSION['otp_status']) || !isset($_SESSION['arr2'])) {
      $response = new RedirectResponse($base_url . '/user/register?q=Access Denied.');
      $response->send();
    }

    \Drupal::service('page_cache_kill_switch')->trigger();
    $form = self::miniorange_otp_login_build_form($form, $form_state);
    return $form;
  }

  /**
   * Custom form build.
   */
  function miniorange_otp_login_build_form($form, &$form_state, $success_form = TRUE)
  {
    $submit_attributes = array();
    $form = self::miniorange_otp_login_build_form_content($form, $form_state, $success_form);
    $form['loader']['#markup'] = '</div><div class="mo_otp-modal-mo_otp_container mo_otp-modal-footer">';

    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Validate',
      '#submit' => array('::miniorange_otp_validation_user_validate'),
    );

    $form['actions']['resend'] = array(
      '#type' => 'submit',
      '#value' => t('Resend OTP'),
      '#attributes' => $submit_attributes,
      '#limit_validation_errors' => array(),
      '#submit' => array('::miniorange_otp_resend_otp'),
    );

    $form['actions']['back'] = array(
      '#type' => 'submit',
      '#value' => t('Back'),
      '#attributes' => $submit_attributes,
      '#limit_validation_errors' => array(),
      '#submit' => array('::miniorange_otp_back'),
    );

    return $form;
  }

  function miniorange_otp_login_build_form_content($form, $form_state, $success_form = TRUE)
  {
    return self::miniorange_otp_login_build_otp_validation_form($form, $form_state, $success_form);
  }

  function miniorange_otp_back($form, $form_state)
  {
    global $base_url;
    $response = new RedirectResponse($base_url . '/user/register');
    $response->send();
  }

  /**
   * Custom form build with error message.
   */
  function miniorange_otp_build_form_with_error_message(&$form_state)
  {
    $form = array();
    $form = self::miniorange_otp_build_form($form, $form_state, FALSE);
    $form_state['complete form']['header']['#markup'] = $form['header']['#markup'];
    return $form;
  }

  /**
   * Custom validation form build.
   */
  public function miniorange_otp_login_build_otp_validation_form($form, &$form_state, $success_message = TRUE)
  {
    $config = \Drupal::config('otp_verification.settings');
    $otp_options = $config->get('miniorange_otp_options');
    $otp_sent = $_SESSION['otp_status'];

    if (isset($_GET['otpto']) && $otp_sent == 1) {
      $code = $_GET['otpto'];
      $msg = self::miniorange_email_phone_success_msg($otp_options, $code);

      \Drupal::messenger()->addMessage(t($msg), 'status');
      $_SESSION['otp_status'] = 0;
    }

    if ($otp_options == 0) {
      $form['otp'] = array(
        '#name' => 'otp',
        '#type' => 'textfield',
        '#attributes' => array('autofocus' => 'autofocus',),
        '#title' => t('OTP'),
        '#default_value' => '',
        '#size' => 60,
        '#description' => t('Please enter the OTP that has been sent to the registered Email ID.'),
        '#maxlength' => 15,
        '#required' => true,
      );
      return $form;
    } elseif ($otp_options == 1) {
      $form['otp'] = array(
        '#name' => 'otp',
        '#type' => 'textfield',
        '#attributes' => array('autofocus' => 'autofocus',),
        '#title' => t('OTP'),
        '#default_value' => '',
        '#size' => 60,
        '#description' => t('Please enter the OTP that has been sent to the registered Phone.'),
        '#maxlength' => 15,
        '#required' => true,
      );
      return $form;
    }
  }

  public function miniorange_email_phone_success_msg($otp_options, $code)
  {
    $config = \Drupal::config('otp_verification.settings');
    if ($otp_options == 0) {
      $success_msg = $config->get('miniorange_success_email_otp_message', '');
      if (!empty($success_msg))
        $success_msg = str_replace('##email##', $code, $success_msg);
      else
        $success_msg = 'A One Time Passcode has been sent to ' . $code . '. Please enter the OTP below to verify your email address. If you cannot see the email in your inbox, make sure to check your SPAM folder.';
    }

    if ($otp_options == 1) {
      $success_msg = $config->get('miniorange_success_phone_otp_message', '');
      if (!empty($success_msg))
        $success_msg = str_replace('##phone##', $code, $success_msg);
      else
        $success_msg = 'A One Time Passcode has been sent to ' . $code . '. Please enter the OTP below to verify your phone number.';
    }

    return $success_msg;
  }

  /**
   * Resend OTP
   */
  function miniorange_otp_resend_otp(&$form, $form_state)
  {

    global $_miniorange_otp_x;
    $db_var = \Drupal::configFactory()->getEditable('otp_verification.settings');
    $config = \Drupal::config('otp_verification.settings');
    $user_values = $_SESSION['arr2'];
    $umail = $user_values['mail'];
    $otp_options = $config->get('miniorange_otp_options');
    \Drupal::messenger()->deleteAll();

    if ($otp_options == 0) {
      $emailcount = mb_strlen($umail);
      $emailc1 = mb_substr($umail, 0, 3);
      $emailc2 = mb_substr($umail, $emailcount - 4, $emailcount);

      for ($i = 4; $i <= $emailcount - 4; $i++) {
        $_miniorange_otp_x = "X" . $_miniorange_otp_x;
      }

      $email_code = $emailc1 . $_miniorange_otp_x . $emailc2;
      if (!MiniorangeOtpUtilities::isCurlInstalled()) {
        return json_encode(array(
          "status" => 'CURL_ERROR',
          "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
        ));
      }

      $url = 'https://login.xecurify.com/moas/api/auth/challenge';
      $ch = curl_init($url);

      $api_key = $config->get('miniorange_otp_verification_customer_api_key');
      $customer_key = $config->get('miniorange_otp_verification_customer_id');

      // Current time in milliseconds since midnight, January 1, 1970 UTC.
      $current_time_in_millis = round(microtime(TRUE) * 1000);

      // Creating the Hash using SHA-512 algorithm.
      $string_to_hash = $customer_key . $current_time_in_millis . $api_key;
      $hash_value = hash("sha512", $string_to_hash);
      $customer_key_header = "Customer-Key: " . $customer_key;
      $timestamp_header = "Timestamp: " . $current_time_in_millis;
      $authorization_header = "Authorization: " . $hash_value;
      $fields = array(
        'customerKey' => $customer_key,
        'email' => $umail,
        'authType' => 'EMAIL',
        'transactionName' => 'Drupal OTP Verification',
      );

      $field_string = json_encode($fields);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
      curl_setopt($ch, CURLOPT_ENCODING, "");
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
      curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array(
        "Content-Type: application/json",
        $customer_key_header,
        $timestamp_header,
        $authorization_header,
      ));

      curl_setopt($ch, CURLOPT_POST, TRUE);
      curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
      $content = curl_exec($ch);
      $trans_status = json_decode($content)->status;
      curl_close($ch);

      if ($trans_status == "SUCCESS") {
        $tx_id = json_decode($content)->txId;
        $_SESSION['trans_id'] = $tx_id;

        \Drupal::messenger()->addMessage(t('An OTP has been resent to @otp_resend', array('@otp_resend' => $email_code)), 'status');
      } else {
        $error_phone_msg = $config->get('miniorange_error_email_otp_message', '');
        if (!empty($error_phone_msg))
          \Drupal::messenger()->addMessage(t($error_phone_msg), 'error');
        else
          \Drupal::messenger()->addMessage(t('There was an error in sending the OTP.'), 'error');
      }
    } else {
      $ph = $_SESSION['phone_during_register'];
      $phno = mb_strlen($ph);
      $phbr1 = mb_substr($ph, 0, 4);
      $phbr2 = mb_substr($ph, $phno - 2, $phno);

      for ($i = 5; $i <= $phno - 2; $i++) {
        $_miniorange_otp_x = "X" . $_miniorange_otp_x;
      }

      $ph_code = $phbr1 . $_miniorange_otp_x . $phbr2;
      if (!MiniorangeOtpUtilities::isCurlInstalled()) {
        return json_encode(array(
          "status" => 'CURL_ERROR',
          "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
        ));
      }

      $url = 'https://login.xecurify.com/moas/api/auth/challenge';
      $ch = curl_init($url);
      $api_key = $config->get('miniorange_otp_verification_customer_api_key');
      $customer_key = $config->get('miniorange_otp_verification_customer_id');
      // Current time in milliseconds since midnight, January 1, 1970 UTC.
      $current_time_in_millis = round(microtime(TRUE) * 1000);
      // Creating the Hash using SHA-512 algorithm.
      $string_to_hash = $customer_key . $current_time_in_millis . $api_key;
      $hash_value = hash("sha512", $string_to_hash);
      $customer_key_header = "Customer-Key: " . $customer_key;
      $timestamp_header = "Timestamp: " . $current_time_in_millis;
      $authorization_header = "Authorization: " . $hash_value;

      $fields = array(
        'customerKey' => $customer_key,
        'phone' => $ph,
        'authType' => 'SMS',
        'transactionName' => 'Drupal OTP Verification',
      );

      $field_string = json_encode($fields);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
      curl_setopt($ch, CURLOPT_ENCODING, "");
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
      curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
      curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", $customer_key_header, $timestamp_header, $authorization_header,));
      curl_setopt($ch, CURLOPT_POST, TRUE);
      curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);

      $content = curl_exec($ch);
      $trans_status = json_decode($content)->status;
      curl_close($ch);
      if ($trans_status == "SUCCESS") {
        $tx_id = json_decode($content)->txId;
        $_SESSION['trans_id'] = $tx_id;
        \Drupal::messenger()->addMessage(t('OTP has been resent to @otp_resend_phone', array('@otp_resend_phone' => $ph_code)), 'status');
      } else {
        $error_phone_msg = $config->get('miniorange_error_phone_otp_message', '');
        if (!empty($error_phone_msg))
          \Drupal::messenger()->addMessage(t($error_phone_msg), 'error');
        else
          \Drupal::messenger()->addMessage(t('There was an error in sending the OTP.'), 'error');
      }
    }
  }

  /**
   * Handling and validating user.
   */
  public function miniorange_otp_validation_user_validate($form, &$form_state)
  {

    $config = \Drupal::config('otp_verification.settings');
    $db_var = \Drupal::configFactory()->getEditable('otp_verification.settings');

    $uname = isset($_SESSION['miniorange_otp']['register_values']['name']) ? $_SESSION['miniorange_otp']['register_values']['name'] : '';
    $otp_options = $config->get('miniorange_otp_options');
    $mail = isset($_SESSION['miniorange_otp']['register_values']['mail']) ? $_SESSION['miniorange_otp']['register_values']['mail'] : '';
    $pwd = isset($_SESSION['miniorange_otp']['register_values']['pass']) ? $_SESSION['miniorange_otp']['register_values']['pass'] : '';

    $otp_token = $form_state->getValue('otp');
    $otp_token = trim($otp_token);

    $trans_id = $_SESSION['trans_id'];

    \Drupal::messenger()->deleteAll();

    if (!MiniorangeOtpUtilities::isCurlInstalled()) {
      return json_encode(array(
        "status" => 'CURL_ERROR',
        "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }

    $url = 'https://login.xecurify.com/moas/api/auth/validate';
    $ch = curl_init($url);
    global $base_url;
    $customer_key = $config->get('miniorange_otp_verification_customer_id');
    $api_key = $config->get('miniorange_otp_verification_customer_api_key');
    $current_time_in_millis = round(microtime(TRUE) * 1000);
    $string_to_hash = $customer_key . $current_time_in_millis . $api_key;
    $hash_value = hash("sha512", $string_to_hash);
    $customer_key_header = "Customer-Key: " . $customer_key;
    $timestamp_header = "Timestamp: " . $current_time_in_millis;
    $authorization_header = "Authorization: " . $hash_value;

    if ($otp_options == 1) {
      $fields = array(
        'txId' => $trans_id,
        'token' => $otp_token,
        'customerKey' => $customer_key,
        'username' => $mail,
        'authType' => 'PHONE',
      );
    } else {
      $fields = array(
        'txId' => $trans_id,
        'token' => $otp_token,
        'customerKey' => $customer_key,
        'username' => $mail,
        'authType' => 'EMAIL',
      );
    }

    $field_string = json_encode($fields);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
    curl_setopt($ch, CURLOPT_ENCODING, "");
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json", $customer_key_header, $timestamp_header, $authorization_header,));
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $field_string);
    $content = curl_exec($ch);
    curl_close($ch);

    $db_var->set('miniorange_otp_token', $otp_token)->save();
    $otp_status = json_decode($content)->status;
    $phone_number = $config->get('phone_number_register');

    \Drupal::messenger()->deleteAll();

    if ($otp_status == "SUCCESS") {

      $arr2 = $_SESSION['arr2'];
      $default_role = "authenticated user";
      unset($_SESSION['miniorange_otp']);

      $machine_name_of_phone_field = $config->get('machine_name_of_phone_field');
    
    $custom_fields = $config->get('miniorange_custom_fields_registration', '');
    $custom_fields_register = isset($custom_fields) ? explode(';', $custom_fields) : '';
    
      $new_user = array(
        'name' => $arr2['name'],
        'mail' => $arr2['mail'],
        'pass' => $arr2['pass'],
        'status' => 1,
        'roles' => $default_role,
        $machine_name_of_phone_field => $phone_number,
      );

    if($custom_fields_register[0] != ''){
        foreach ($custom_fields_register as $custom) {
            $new_user[$custom] = $arr2[$custom][0]['value'];
        }
    }

      $account = User::create($new_user);
      $account->save();

      !empty(\Drupal::config('otp_verification.settings')->get('otp_login_url')) ? $edit['redirect'] = \Drupal::config('otp_verification.settings')->get('otp_login_url') : $edit['redirect'] = $base_url;
      user_login_finalize($account);
      \Drupal::messenger()->addMessage(t('Registration successful. Now you are logged in.'), 'status');
      $response = new RedirectResponse($edit['redirect']);
      $response->send();
    } elseif ($otp_status == "FAILED") {
      $invalid_otp_msg = $config->get('miniorange_invalid_otp_message', '');
      if (!empty($invalid_otp_msg))
        \Drupal::messenger()->addMessage(t($invalid_otp_msg), 'error');
      else
        \Drupal::messenger()->addMessage(t('OTP entered is incorrect. Please enter valid OTP.'), 'error');
    }
  }

  public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state)
  {

  }
}
